@@ -17,8 +17,8 @@
 
 ''' The main Barman module
 '''
-
-from version import __version__
+from __future__ import absolute_import
+from .version import __version__
 
 __config__ = None
 
