@@ -1,4 +1,4 @@
-# Copyright (C) 2011, 2012 2ndQuadrant Italia (Devise.IT S.r.L.)
+# Copyright (C) 2011-2013 2ndQuadrant Italia (Devise.IT S.r.L.)
 #
 # This file is part of Barman.
 #
@@ -15,9 +15,9 @@
 # You should have received a copy of the GNU General Public License
 # along with Barman.  If not, see <http://www.gnu.org/licenses/>.
 
-''' This module represents a bakup. '''
+''' This module represents a backup. '''
 
-from barman import xlog, _pretty_size
+from barman import xlog, _pretty_size, version
 from barman.command_wrappers import RsyncPgData, Command
 from barman.compression import CompressionManager, CompressionIncompatibility
 from glob import glob
@@ -60,11 +60,19 @@ class BackupInfo(object):
              'begin_time':dateutil.parser.parse,
              'end_time':dateutil.parser.parse,
              'size':int,
+             'version':int,
     }
     '''Conversion from string '''
     TYPES_OUT = {'tablespaces':repr, # Treat the tablespaces as a literal Python list of tuples
     }
 
+    '''Status according to retention policies'''
+    OBSOLETE = 'OBSOLETE'
+    VALID = 'VALID'
+    POTENTIALLY_OBSOLETE = 'OBSOLETE*'
+    NONE = '-'
+    RETENTION_STATUS = (OBSOLETE, VALID, POTENTIALLY_OBSOLETE, NONE)
+
     def __init__(self, server, info_file=None, backup_id=None):
         '''Constructor '''
         # Initialises the attributes for the object based on the predefined keys
@@ -112,7 +120,7 @@ class BackupInfo(object):
 
     def get_required_wal_segments(self):
         '''Get the list of required WAL segments for the current backup'''
-        return xlog.enumerate_segments(self.begin_wal, self.end_wal)
+        return xlog.enumerate_segments(self.begin_wal, self.end_wal, self.version)
 
     def get_list_of_files(self, target):
         '''Get the list of files for the current backup'''
@@ -144,7 +152,7 @@ class BackupInfo(object):
                 next_backup = self.backup_manager.get_next_backup(self.backup_id)
                 wal_num, wal_size, wal_until_next_num, wal_until_next_size, wal_last = self.server.get_wal_info(self)
                 yield "  Server Name       : %s" % self.server_name
-                yield "  Status:           : %s" % self.status
+                yield "  Status            : %s" % self.status
                 yield "  PostgreSQL Version: %s" % self.version
                 yield "  PGDATA directory  : %s" % self.pgdata
                 if self.tablespaces:
@@ -171,6 +179,10 @@ class BackupInfo(object):
                 yield "    Last available  : %s" % wal_last
                 yield ""
                 yield "  Catalog information:"
+                if self.server.enforce_retention_policies:
+                    yield "    Retention Policy: %s" % self.server.config.retention_policy.backup_status(self.backup_id)
+                else:
+                    yield "    Retention Policy: not enforced"
                 if previous_backup:
                     yield "    Previous Backup : %s" % previous_backup.backup_id
                 else:
@@ -243,7 +255,7 @@ class BackupManager(object):
         self.config = server.config
         self.available_backups = {}
         self.compression_manager = CompressionManager(self.config)
-        
+
         # used for error messages
         self.current_action = None
 
@@ -254,7 +266,7 @@ class BackupManager(object):
         :param status_filter: default DEFAULT_STATUS_FILTER. The status of the backup list returned
         '''
         if not isinstance(status_filter, tuple):
-            status_filter = tuple(status_filter)
+            status_filter = tuple(status_filter,)
         if status_filter not in self.available_backups:
             available_backups = {}
             for filename in glob("%s/*/backup.info" % self.config.basebackups_directory):
@@ -345,6 +357,15 @@ class BackupManager(object):
 
         :param backup: the backup to delete
         '''
+        available_backups = self.get_available_backups()
+        # Honour minimum required redundancy
+        if backup.status == BackupInfo.DONE and self.server.config.minimum_redundancy >= len(available_backups):
+            yield "Skipping delete of backup %s for server %s due to minimum redundancy requirements (%s)" % (
+                backup.backup_id, self.config.name, self.server.config.minimum_redundancy)
+            _logger.warning("Could not delete backup %s for server %s - minimum redundancy = %s, current size = %s"
+                % (backup.backup_id, self.config.name, self.server.config.minimum_redundancy, len(available_backups)))
+            return
+
         yield "Deleting backup %s for server %s" % (backup.backup_id, self.config.name)
         previous_backup = self.get_previous_backup(backup.backup_id)
         next_backup = self.get_next_backup(backup.backup_id)
@@ -385,6 +406,7 @@ class BackupManager(object):
         env['BARMAN_PHASE'] = phase
         env['BARMAN_STATUS'] = backup_info.status
         env['BARMAN_ERROR'] = backup_info.error or ''
+        env['BARMAN_VERSION'] = version.__version__
         return env
 
     def run_pre_backup_script(self, backup_info):
@@ -482,7 +504,7 @@ class BackupManager(object):
 
             msg = "Backup failed %s" % self.current_action
             _logger.exception(msg)
-            yield msg
+            raise Exception("ERROR: %s" % msg)
 
         else:
             msg = "Backup end at xlog location: %s (%s, %08X)" % (backup_info.end_xlog,
@@ -500,7 +522,7 @@ class BackupManager(object):
             self.run_post_backup_script(backup_info)
 
 
-    def recover(self, backup, dest, tablespaces, target_tli, target_time, target_xid, exclusive, remote_command):
+    def recover(self, backup, dest, tablespaces, target_tli, target_time, target_xid, target_name, exclusive, remote_command):
         '''
         Performs a recovery of a backup
 
@@ -510,7 +532,8 @@ class BackupManager(object):
         :param target_tli: the target timeline
         :param target_time: the target time
         :param target_xid: the target xid
-        :param exclusive: whether the recovery is exlusive or not
+        :param target_name: the target name created previously with pg_create_restore_point() function call
+        :param exclusive: whether the recovery is exclusive or not
         :param remote_command: default None. The remote command to recover the base backup,
                                in case of remote backup.
         '''
@@ -520,7 +543,7 @@ class BackupManager(object):
         recovery_dest = 'local'
         if remote_command:
             recovery_dest = 'remote'
-            rsync = RsyncPgData(ssh=remote_command)
+            rsync = RsyncPgData(ssh=remote_command, bwlimit=self.config.bandwidth_limit)
         msg = "Starting %s restore for server %s using backup %s " % (recovery_dest, self.config.name, backup.backup_id)
         yield msg
         _logger.info(msg)
@@ -549,7 +572,7 @@ class BackupManager(object):
                             os.unlink(location)
                         if not os.path.exists(location):
                             os.makedirs(location)
-                        # test permissiones
+                        # test permissions
                         barman_write_check_file = os.path.join(location, '.barman_write_check')
                         file(barman_write_check_file, 'a').close()
                         os.unlink(barman_write_check_file)
@@ -559,7 +582,10 @@ class BackupManager(object):
                         _logger.critical(msg)
                         raise SystemExit(msg)
                     yield "\t%s, %s, %s" % (oid, name, location)
+
+        wal_dest = os.path.join(dest, 'pg_xlog')
         target_epoch = None
+        target_datetime = None
         if target_time:
             try:
                 target_datetime = dateutil.parser.parse(target_time)
@@ -568,7 +594,7 @@ class BackupManager(object):
                 _logger.critical(msg)
                 raise SystemExit(msg)
             target_epoch = time.mktime(target_datetime.timetuple()) + (target_datetime.microsecond / 1000000.)
-        if target_time or target_xid or (target_tli and target_tli != backup.timeline):
+        if target_time or target_xid or (target_tli and target_tli != backup.timeline) or target_name:
             targets = {}
             if target_time:
                 targets['time'] = str(target_datetime)
@@ -576,8 +602,11 @@ class BackupManager(object):
                 targets['xid'] = str(target_xid)
             if target_tli and target_tli != backup.timeline:
                 targets['timeline'] = str(target_tli)
+            if target_name:
+                targets['name'] = str(target_name)
             yield "Doing PITR. Recovery target %s" % \
                 (", ".join(["%s: %r" % (k, v) for k, v in targets.items()]))
+            wal_dest = os.path.join(dest, 'barman_xlog')
 
         # Copy the base backup
         msg = "Copying the base backup."
@@ -590,13 +619,10 @@ class BackupManager(object):
         msg = "Copying required wal segments."
         _logger.info(msg)
         yield msg
-        if target_time or target_xid or (target_tli and target_tli != backup.timeline):
-            wal_dest = os.path.join(dest, 'barman_xlog')
-        else:
-            wal_dest = os.path.join(dest, 'pg_xlog')
+
         # Retrieve the list of required WAL segments according to recovery options
         xlogs = {}
-        required_xlog_files = tuple(self.server.get_required_xlog_files(backup, target_tli, target_epoch, target_xid))
+        required_xlog_files = tuple(self.server.get_required_xlog_files(backup, target_tli, target_epoch))
         for filename in required_xlog_files:
             hashdir = xlog.hash_dir(filename)
             if hashdir not in xlogs:
@@ -610,7 +636,7 @@ class BackupManager(object):
         _logger.info("Wal segmets copied.")
 
         # Generate recovery.conf file (only if needed by PITR)
-        if target_time or target_xid or (target_tli and target_tli != backup.timeline):
+        if target_time or target_xid or (target_tli and target_tli != backup.timeline) or target_name:
             msg = "Generating recovery.conf"
             yield  msg
             _logger.info(msg)
@@ -627,8 +653,10 @@ class BackupManager(object):
                 print >> recovery, "recovery_target_timeline = %s" % target_tli
             if target_xid:
                 print >> recovery, "recovery_target_xid = '%s'" % target_xid
-                if exclusive:
-                    print >> recovery, "recovery_target_inclusive = '%s'" % (not exclusive)
+            if target_name:
+                print >> recovery, "recovery_target_name = '%s'" % target_name
+            if (target_xid or target_time) and exclusive:
+                print >> recovery, "recovery_target_inclusive = '%s'" % (not exclusive)
             recovery.close()
             if remote_command:
                 recovery = rsync.from_file_list(['recovery.conf'], tempdir, ':%s' % dest)
@@ -642,7 +670,7 @@ class BackupManager(object):
                 status_dir = os.path.join(wal_dest, 'archive_status')
                 os.makedirs(status_dir) # no need to check, it must not exist
             for filename in required_xlog_files:
-                with file(os.path.join(status_dir, "%s.done" % filename), 'a') as f:
+                with open(os.path.join(status_dir, "%s.done" % filename), 'a') as f:
                     f.write('')
             if remote_command:
                 retval = rsync('%s/' % status_dir, ':%s' % os.path.join(wal_dest, 'archive_status'))
@@ -726,6 +754,16 @@ class BackupManager(object):
         if not found and verbose:
             yield "\tno file found"
 
+        # Retention policy management
+        if self.server.enforce_retention_policies and self.config.retention_policy_mode == 'auto':
+            available_backups = self.get_available_backups(BackupInfo.STATUS_ALL)
+            retention_status = self.config.retention_policy.report()
+            for bid in sorted(retention_status.iterkeys()):
+                if retention_status[bid] == BackupInfo.OBSOLETE:
+                    _logger.info("Enforcing retention policy: removing backup %s for server %s" % (
+                        bid, self.config.name))
+                    for line in self.delete_backup(available_backups[bid]):
+                        yield line
 
     #
     # Hooks
@@ -781,7 +819,7 @@ class BackupManager(object):
             for key in sorted(cf.keys()):
                 backup_info.set_attribute(key, cf[key])
 
-        # Get server version and tablespaces information
+        # Get tablespaces information
         self.current_action = "detecting tablespaces"
         _logger.debug(self.current_action)
         tablespaces = self.server.get_pg_tablespaces()
@@ -813,14 +851,46 @@ class BackupManager(object):
 
         :param backup_info: the backup information structure
         '''
+
+        # validate the bandwidth rules against the tablespace list
+        tablespaces_bwlimit={}
+        if self.config.tablespace_bandwidth_limit and backup_info.tablespaces:
+            valid_tablespaces = dict([(tablespace_data[0], tablespace_data[1])
+                                     for tablespace_data in backup_info.tablespaces])
+            for tablespace, bwlimit in self.config.tablespace_bandwidth_limit.items():
+                if tablespace in valid_tablespaces:
+                    tablespace_dir = "pg_tblspc/%s" % (valid_tablespaces[tablespace],)
+                    tablespaces_bwlimit[tablespace_dir] = bwlimit
+
         backup_dest = os.path.join(backup_info.get_basebackup_directory(), 'pgdata')
-        rsync = RsyncPgData(ssh=self.server.ssh_command, ssh_options=self.server.ssh_options)
+        rsync = RsyncPgData(ssh=self.server.ssh_command,
+                ssh_options=self.server.ssh_options,
+                bwlimit=self.config.bandwidth_limit,
+                exclude_and_protect=tablespaces_bwlimit.keys())
         retval = rsync(':%s/' % backup_info.pgdata, backup_dest)
         if retval not in (0, 24):
             msg = "ERROR: data transfer failure"
             _logger.exception(msg)
             raise Exception(msg)
 
+        # deal with tablespaces with a different bwlimit
+        if len(tablespaces_bwlimit) > 0:
+            for tablespace_dir, bwlimit in tablespaces_bwlimit.items():
+                self.current_action = "copying tablespace '%s' with bwlimit %d" % (
+                    tablespace_dir, bwlimit)
+                _logger.debug(self.current_action)
+                tb_rsync = RsyncPgData(ssh=self.server.ssh_command,
+                    ssh_options=self.server.ssh_options,
+                    bwlimit=bwlimit)
+                retval = tb_rsync(
+                    ':%s/' % os.path.join(backup_info.pgdata, tablespace_dir),
+                    os.path.join(backup_dest, tablespace_dir))
+                if retval not in (0, 24):
+                    msg = "ERROR: data transfer failure on directory '%s'" % (
+                        tablespace_dir,)
+                    _logger.exception(msg)
+                    raise Exception(msg)
+
         # Copy configuration files (if not inside PGDATA)
         self.current_action = "copying configuration files"
         _logger.debug(self.current_action)
@@ -870,13 +940,44 @@ class BackupManager(object):
         :param remote_command: default None. The remote command to recover the base backup,
                                in case of remote backup.
         '''
-        rsync = RsyncPgData(ssh=remote_command)
-        sourcedir = '%s/' % os.path.join(backup.get_basebackup_directory(), 'pgdata')
+
+        sourcedir = os.path.join(backup.get_basebackup_directory(), 'pgdata')
+        tablespaces_bwlimit={}
         if remote_command:
             dest = ':%s' % dest
-        retval = rsync(sourcedir, dest)
+
+            # validate the bandwidth rules against the tablespace list
+            if self.config.tablespace_bandwidth_limit and backup.tablespaces:
+                valid_tablespaces = dict([(tablespace_data[0], tablespace_data[1])
+                                          for tablespace_data in backup.tablespaces])
+                for tablespace, bwlimit in self.config.tablespace_bandwidth_limit.items():
+                    if tablespace in valid_tablespaces:
+                        tablespace_dir = "pg_tblspc/%s" % (valid_tablespaces[tablespace],)
+                        tablespaces_bwlimit[tablespace_dir] = bwlimit
+
+        rsync = RsyncPgData(ssh=remote_command,
+                bwlimit=self.config.bandwidth_limit,
+                exclude_and_protect=tablespaces_bwlimit.keys())
+        retval = rsync('%s/' % (sourcedir,), dest)
         if retval != 0:
             raise Exception("ERROR: data transfer failure")
+
+        if remote_command and len(tablespaces_bwlimit) > 0:
+            for tablespace_dir, bwlimit in tablespaces_bwlimit.items():
+                self.current_action = "copying tablespace '%s' with bwlimit %d" % (
+                    tablespace_dir, bwlimit)
+                _logger.debug(self.current_action)
+                tb_rsync = RsyncPgData(ssh=remote_command,
+                                       bwlimit=bwlimit)
+                retval = tb_rsync(
+                    '%s/' % os.path.join(sourcedir, tablespace_dir),
+                    os.path.join(dest, tablespace_dir))
+                if retval != 0:
+                    msg = "ERROR: data transfer failure on directory '%s'" % (
+                        tablespace_dir,)
+                    _logger.exception(msg)
+                    raise Exception(msg)
+
         # TODO: Manage different location for configuration files that were not within the data directory
 
     def recover_xlog_copy(self, decompressor, xlogs, wal_dest, remote_command=None):
@@ -889,7 +990,7 @@ class BackupManager(object):
         :param remote_command: default None. The remote command to recover the xlog,
                                in case of remote backup.
         '''
-        rsync = RsyncPgData(ssh=remote_command)
+        rsync = RsyncPgData(ssh=remote_command, bwlimit=self.config.bandwidth_limit)
         if remote_command:
             # If remote recovery tell rsync to copy them remotely
             wal_dest = ':%s' % wal_dest
@@ -962,6 +1063,16 @@ class BackupManager(object):
 
             yield ("\tcompression settings: %s" % status, status == 'OK')
 
+        # Minimum redundancy checks
+        no_backups = len(self.get_available_backups())
+        if (no_backups < self.config.minimum_redundancy):
+            status = 'FAILED'
+        else:
+            status = 'OK'
+        yield ("\tminimum redundancy requirements: %s (have %s backups, expected at least %s)" % (
+                status, no_backups, self.config.minimum_redundancy), status == 'OK')
+
+
     def status(self):
         '''This function show the server status '''
         no_backups = len(self.get_available_backups())
@@ -971,6 +1082,9 @@ class BackupManager(object):
         elif no_backups > 1:
             yield "\tfirst available backup: %s" % self.get_first_backup()
             yield "\tlast available backup: %s" % self.get_last_backup()
+        if (no_backups < self.config.minimum_redundancy):
+            yield "\tYou are currently below the minimum redundancy requirements: %s/%s" % (
+                no_backups, self.config.minimum_redundancy)
 
     def pg_config_mangle(self, filename, settings, backup_filename=None):
         '''This method modifies the postgres configuration file,
