@@ -1,4 +1,4 @@
-# Copyright (C) 2011, 2012 2ndQuadrant Italia (Devise.IT S.r.L.)
+# Copyright (C) 2011-2013 2ndQuadrant Italia (Devise.IT S.r.L.)
 #
 # This file is part of Barman.
 #
@@ -19,7 +19,7 @@
 and the logger.
 '''
 
-from argh import ArghParser, alias, arg
+from argh import ArghParser, named, arg, expects_obj
 from barman.lockfile import lockfile
 from barman.server import Server
 import barman.config
@@ -30,8 +30,9 @@ from barman.backup import BackupInfo
 
 _logger = logging.getLogger(__name__)
 
-@alias('list-server')
+@named('list-server')
 @arg('--minimal', help='machine readable output', action='store_true')
+@expects_obj
 def list_server(args):
     """ List available servers, with useful information
     """
@@ -42,6 +43,7 @@ def list_server(args):
         else:
             yield name
 
+@expects_obj
 def cron(args):
     """ Run maintenance tasks
     """
@@ -56,59 +58,111 @@ def cron(args):
                 for lines in server.cron(verbose=True):
                     yield lines
 
-@arg('server_name', nargs='+', help="specifies the server names for the backup command ('all' will show all available servers)")
+def server_completer(prefix, parsed_args, **kwargs):
+    global_config(parsed_args)
+    for conf in barman.__config__.servers():
+         if conf.name.startswith(prefix):
+             yield conf.name
+
+def server_completer_all(prefix, parsed_args, **kwargs):
+    print >> sys.stderr,  "INVOKED"
+    global_config(parsed_args)
+    for conf in barman.__config__.servers():
+        if conf.name.startswith(prefix):
+            yield conf.name
+    if 'all'.startswith(prefix):
+        yield 'all'
+
+def backup_completer(prefix, parsed_args, **kwargs):
+    global_config(parsed_args)
+    server = get_server(parsed_args)
+    if server:
+        for backup_id in sorted(server.get_available_backups().iterkeys(), reverse=True):
+            if backup_id.startswith(prefix):
+                 yield backup_id
+    else:
+        return
+
+@arg('server_name', nargs='+',
+     completer=server_completer_all,
+     help="specifies the server names for the backup command ('all' will show all available servers)")
+@expects_obj
 def backup(args):
     """ Perform a full backup for the given server
     """
     servers = get_server_list(args)
+    ok = True
     for name in sorted(servers):
         server = servers[name]
         if server == None:
             yield "Unknown server '%s'" % (name)
+            ok = False
             continue
-        for line in server.backup():
-            yield line
+        try:
+            for line in server.backup():
+                yield line
+        except:
+            ok = False
         yield ''
+    if not ok:
+        raise SystemExit(1)
 
-@alias('list-backup')
-@arg('server_name', nargs='+', help="specifies the server name for the command ('all' will show all available servers)")
+@named('list-backup')
+@arg('server_name', nargs='+',
+     completer=server_completer_all,
+     help="specifies the server name for the command ('all' will show all available servers)")
 @arg('--minimal', help='machine readable output', action='store_true')
+@expects_obj
 def list_backup(args):
     """ List available backups for the given server (supports 'all')
     """
     servers = get_server_list(args)
+    ok = True
     for name in sorted(servers):
         server = servers[name]
         if server == None:
             yield "Unknown server '%s'" % (name)
-            return
+            ok = False
+            continue
         if not args.minimal:
             for line in server.list_backups():
                 yield line
         else:
             for backup_id in sorted(server.get_available_backups().iterkeys(), reverse=True):
                 yield backup_id
+    if not ok:
+        raise SystemExit(1)
 
-@arg('server_name', nargs='+', help='specifies the server name for the command')
+@arg('server_name', nargs='+',
+     completer=server_completer_all,
+     help='specifies the server name for the command')
+@expects_obj
 def status(args):
     """ Shows live information and status of the PostgreSQL server
     """
     servers = get_server_list(args)
+    ok = True
     for name in sorted(servers):
         server = servers[name]
         if server == None:
             yield "Unknown server '%s'" % (name)
+            ok = False
             continue
         for line in server.status():
             yield line
         yield ''
+    if not ok:
+        raise SystemExit(1)
 
 
-@arg('server_name', help='specifies the server name for the command')
+@arg('server_name',
+     completer=server_completer,
+     help='specifies the server name for the command')
 @arg('--target-tli', help='target timeline', type=int)
 @arg('--target-time',
      help='target time. You can use any valid unambiguous representation. e.g: "YYYY-MM-DD HH:MM:SS.mmm"')
-@arg('--target-xid', help='target xid')
+@arg('--target-xid', help='target transaction ID')
+@arg('--target-name', help='target name created previously with pg_create_restore_point() function call')
 @arg('--exclusive',
      help='set target xid to be non inclusive', action="store_true")
 @arg('--tablespace',
@@ -123,6 +177,7 @@ def status(args):
      help='specifies the backup ID to recover')
 @arg('destination_directory',
      help='the directory where the new server is created')
+@expects_obj
 def recover(args):
     """ Recover a server at a given time or xid
     """
@@ -163,32 +218,21 @@ def recover(args):
                                target_tli=args.target_tli,
                                target_time=args.target_time,
                                target_xid=args.target_xid,
+                               target_name=args.target_name,
                                exclusive=args.exclusive,
                                remote_command=args.remote_ssh_command
                                ):
         yield line
 
-@alias('show-server')
-@arg('server_name', nargs='+', help="specifies the server names to show ('all' will show all available servers)")
+@named('show-server')
+@arg('server_name', nargs='+',
+     completer=server_completer_all,
+     help="specifies the server names to show ('all' will show all available servers)")
+@expects_obj
 def show_server(args):
     """ Show all configuration parameters for the specified servers
     """
     servers = get_server_list(args)
-    for name in sorted(servers):
-        server = servers[name]
-        if server == None:
-            yield "Unknown server '%s'" % (name)
-            continue
-        for line in server.show():
-            yield line
-        yield ''
-
-@arg('server_name', nargs='+', help="specifies the server names to check ('all' will check all available servers)")
-def check(args):
-    """ Check if the server configuration is working.
-    This function returns 0 if every checks pass, or 0 if any of these fails
-    """
-    servers = get_server_list(args)
     ok = True
     for name in sorted(servers):
         server = servers[name]
@@ -196,16 +240,58 @@ def check(args):
             yield "Unknown server '%s'" % (name)
             ok = False
             continue
-        for line, status in server.check():
-            ok &= status
+        for line in server.show():
             yield line
         yield ''
     if not ok:
         raise SystemExit(1)
 
-@alias('show-backup')
-@arg('server_name', help='specifies the server name for the command')
-@arg('backup_id', help='specifies the backup ID')
+@arg('server_name', nargs='+',
+     completer=server_completer_all,
+     help="specifies the server names to check ('all' will check all available servers)")
+@arg('--nagios', help='Nagios plugin compatible output', action='store_true')
+@expects_obj
+def check(args):
+    """ Check if the server configuration is working.
+    This function returns 0 if every checks pass, or 0 if any of these fails
+    """
+    servers = get_server_list(args)
+    issues = 0
+    for name in sorted(servers):
+        ok = True
+        server = servers[name]
+        if server == None:
+            if not args.nagios:
+                yield "Unknown server '%s'" % (name)
+            issues += 1
+            continue
+        for line, status in server.check():
+            ok &= status
+            if not args.nagios:
+                yield line
+        if not args.nagios:
+            yield ''
+        if not ok:
+            issues += 1
+
+    if args.nagios:
+        if issues:
+            yield "BARMAN CRITICAL - %d server out of %d that have been checked has issues" % (issues, len(servers))
+            raise SystemExit(2)
+        else:
+            yield "BARMAN OK - Ready to serve the Espresso backup"
+
+    if issues:
+        raise SystemExit(1)
+
+@named('show-backup')
+@arg('server_name',
+     completer=server_completer,
+     help='specifies the server name for the command')
+@arg('backup_id',
+     completer=backup_completer,
+     help='specifies the backup ID')
+@expects_obj
 def show_backup(args):
     """ This method Shows a single backup information
     """
@@ -222,9 +308,13 @@ def show_backup(args):
     for line in backup.show():
         yield line
 
-@alias('list-files')
-@arg('server_name', help='specifies the server name for the command')
-@arg('backup_id', help='specifies the backup ID')
+@named('list-files')
+@arg('server_name',
+     completer=server_completer,
+     help='specifies the server name for the command')
+@arg('backup_id',
+     completer=backup_completer,
+     help='specifies the backup ID')
 @arg('--target', choices=('standalone', 'data', 'wal', 'full'), default='standalone',
      help='''
      Possible values are: data (just the data files), standalone (base backup files, including required WAL files),
@@ -232,6 +322,7 @@ def show_backup(args):
      full (same as data + wal). Defaults to %(default)s
      '''
      )
+@expects_obj
 def list_files(args):
     """ List all the files for a single backup
     """
@@ -248,8 +339,13 @@ def list_files(args):
     for line in backup.get_list_of_files(args.target):
         yield line
 
-@arg('server_name', help='specifies the server name for the command')
-@arg('backup_id', help='specifies the backup ID')
+@arg('server_name',
+     completer=server_completer,
+     help='specifies the server name for the command')
+@arg('backup_id',
+     completer=backup_completer,
+     help='specifies the backup ID')
+@expects_obj
 def delete(args):
     """ Delete a backup
     """
