@@ -1,4 +1,4 @@
-# Copyright (C) 2011, 2012 2ndQuadrant Italia (Devise.IT S.r.L.)
+# Copyright (C) 2011-2013 2ndQuadrant Italia (Devise.IT S.r.L.)
 #
 # This file is part of Barman.
 #
@@ -29,6 +29,8 @@ from glob import iglob
 
 _logger = logging.getLogger(__name__)
 
+FORBIDDEN_SERVER_NAMES = ['all']
+
 
 class Server(object):
     '''This class represents a server.'''
@@ -36,14 +38,16 @@ class Server(object):
         'backup_directory', 'basebackups_directory',
         'wals_directory', 'incoming_wals_directory', 'lock_file',
         'compression', 'custom_compression_filter',
-        'custom_decompression_filter', 'retention_policy',
+        'custom_decompression_filter', 'retention_policy_mode', 'retention_policy',
         'wal_retention_policy', 'pre_backup_script', 'post_backup_script',
+        'minimum_redundancy', 'bandwidth_limit', 'tablespace_bandwidth_limit'
     ]
 
     BARMAN_KEYS = ['compression', 'custom_compression_filter',
-        'custom_decompression_filter', 'retention_policy',
+        'custom_decompression_filter', 'retention_policy_mode', 'retention_policy',
         'wal_retention_policy', 'pre_backup_script', 'post_backup_script',
         'configuration_files_directory',
+        'minimum_redundancy', 'bandwidth_limit', 'tablespace_bandwidth_limit'
     ]
 
     DEFAULTS = {
@@ -53,6 +57,9 @@ class Server(object):
         'wals_directory': r'%(backup_directory)s/wals',
         'incoming_wals_directory': r'%(backup_directory)s/incoming',
         'lock_file': r'%(backup_directory)s/%(name)s.lock',
+        'retention_policy_mode' : 'auto',
+        'wal_retention_policy' : 'main',
+        'minimum_redundancy' : '0'
     }
 
     def __init__(self, config, name):
@@ -71,10 +78,12 @@ class Server(object):
 class Config(object):
     ''' This class represents the barman configuration.
 
-    Default configuration files are /etc/barman.conf
+    Default configuration files are /etc/barman.conf,
+    /etc/barman/barman.conf
     and ~/.barman.conf for a per-user configuration
     '''
-    CONFIG_FILES = ['~/.barman.conf', '/etc/barman.conf']
+    CONFIG_FILES = ['~/.barman.conf', '/etc/barman.conf',
+            '/etc/barman/barman.conf']
 
     def __init__(self, filename=None):
         self._config = ConfigParser()
@@ -101,7 +110,7 @@ class Config(object):
         if not self._config.has_section(section):
             return None
         try:
-            value = self._config.get(section, option, False, defaults)
+            value = self._config.get(section, option, raw=False, vars=defaults)
             if value == 'None':
                 value = None
             if value != None:
@@ -134,7 +143,7 @@ class Config(object):
         formatter = logging.Formatter(fmt)
         handler.setFormatter(formatter)
         logging.root.addHandler(handler)
-        if warn: _logger.warn(warn) # this will be always displayed because the default level is WARNING
+        if warn: _logger.warn(warn)  # this will be always displayed because the default level is WARNING
         level = self.get('barman', 'log_level') or 'INFO'
         if level.isdigit():
             level_int = int(level)
@@ -177,27 +186,27 @@ class Config(object):
         configuration_files_directory = self.get('barman', 'configuration_files_directory')
 
         if not configuration_files_directory:
-           return
+            return
 
         if not os.path.isdir(os.path.expanduser(configuration_files_directory)):
-           _logger.warn('Ignoring the "configuration_files_directory" option as "%s" is not a directory',
+            _logger.warn('Ignoring the "configuration_files_directory" option as "%s" is not a directory',
                 configuration_files_directory)
-           return
+            return
 
         for cfile in sorted(iglob(os.path.join(os.path.expanduser(configuration_files_directory),
             '*.conf'))):
-           filename = os.path.basename(cfile)
-           if os.path.isfile(cfile):
-               # Load a file
-               _logger.info('Including configuration file: %s', filename)
-               self._config.read(cfile)
-               if self._is_global_config_changed():
-                   msg="the configuration file %s contains a not empty [barman] section" % filename
-                   _logger.fatal(msg)
-                   raise SystemExit("FATAL: %s" % msg)
-           else:
-               # Add an info that a file has been discarded
-               _logger.warn('Discarding configuration file: %s (not a file)', filename)
+            filename = os.path.basename(cfile)
+            if os.path.isfile(cfile):
+                # Load a file
+                _logger.info('Including configuration file: %s', filename)
+                self._config.read(cfile)
+                if self._is_global_config_changed():
+                    msg = "the configuration file %s contains a not empty [barman] section" % filename
+                    _logger.fatal(msg)
+                    raise SystemExit("FATAL: %s" % msg)
+            else:
+                # Add an info that a file has been discarded
+                _logger.warn('Discarding configuration file: %s (not a file)', filename)
 
     def _populate_servers(self):
         '''Populate server list from configuration file'''
@@ -206,7 +215,11 @@ class Config(object):
         self._servers = {}
         for section in self._config.sections():
             if section == 'barman':
-                continue # skip global settings
+                continue  # skip global settings
+            if section in FORBIDDEN_SERVER_NAMES:
+                msg = "the reserved word '%s' is not allowed as server name. Please rename it." % section
+                _logger.fatal(msg)
+                raise SystemExit("FATAL: %s" % msg)
             self._servers[section] = Server(self, section)
 
     def server_names(self):
