@@ -1,4 +1,4 @@
-# Copyright (C) 2011, 2012 2ndQuadrant Italia (Devise.IT S.r.L.)
+# Copyright (C) 2011-2013 2ndQuadrant Italia (Devise.IT S.r.L.)
 #
 # This file is part of Barman.
 #
@@ -69,8 +69,8 @@ class lockfile(object):
             fd = os.open(self.filename, os.O_TRUNC | os.O_CREAT | os.O_RDWR, 0600)
             flags = fcntl.LOCK_EX
             if not wait or (wait == None and not self.wait): flags |= fcntl.LOCK_NB
-            fcntl.flock (fd, flags)
-            os.write(fd, "%s\n" % os.getpid())
+            fcntl.flock(fd, flags)
+            os.write(fd, ("%s\n" % os.getpid()).encode('ascii'))
             self.fd = fd
             return True
         except (OSError, IOError), e:
@@ -91,7 +91,7 @@ class lockfile(object):
         '''
         if not self.fd: return
         try:
-            os.unlink(self.filename)
+            fcntl.flock(self.fd, fcntl.LOCK_UN)
             os.close(self.fd)
         except (OSError, IOError):
             pass
