@@ -1,4 +1,4 @@
-# Copyright (C) 2011, 2012 2ndQuadrant Italia (Devise.IT S.r.L.)
+# Copyright (C) 2011-2013 2ndQuadrant Italia (Devise.IT S.r.L.)
 #
 # This file is part of Barman.
 #
@@ -23,11 +23,15 @@ from barman import xlog, _pretty_size
 from barman.lockfile import lockfile
 from barman.backup import BackupInfo, BackupManager
 from barman.command_wrappers import Command
+from barman.retention_policies import RetentionPolicyFactory, SimpleWALRetentionPolicy
 import os
+import logging
 import psycopg2
 from contextlib import contextmanager
 import itertools
 
+_logger = logging.getLogger(__name__)
+
 class Server(object):
     '''This class represents a server to backup'''
     XLOG_DB = "xlog.db"
@@ -46,6 +50,97 @@ class Server(object):
         self.ssh_options.extend("-o BatchMode=yes -o StrictHostKeyChecking=no".split())
         self.backup_manager = BackupManager(self)
         self.configuration_files = None
+        self.enforce_retention_policies = False
+
+        # Set bandwidth_limit
+        if self.config.bandwidth_limit:
+            try:
+                self.config.bandwidth_limit = int(self.config.bandwidth_limit)
+            except:
+                _logger.warning('Invalid bandwidth_limit "%s" for server "%s" (fallback to "0")'
+                                % (self.config.bandwidth_limit, self.config.name))
+                self.config.bandwidth_limit = None
+
+        # set tablespace_bandwidth_limit
+        if self.config.tablespace_bandwidth_limit:
+            rules = {}
+            for rule in self.config.tablespace_bandwidth_limit.split():
+                try:
+                    key, value = rule.split(':', 1)
+                    value = int(value)
+                    if value != self.config.bandwidth_limit:
+                        rules[key] = value
+                except:
+                    _logger.warning("Invalid tablespace_bandwidth_limit rule '%s'" % (rule,))
+            if len(rules) > 0:
+                self.config.tablespace_bandwidth_limit = rules
+            else:
+                self.config.tablespace_bandwidth_limit = None
+
+        # Set minimum redundancy (default 0)
+        if self.config.minimum_redundancy.isdigit():
+            self.config.minimum_redundancy = int(self.config.minimum_redundancy)
+            if self.config.minimum_redundancy < 0:
+                _logger.warning('Negative value of minimum_redundancy "%s" for server "%s" (fallback to "0")'
+                            % (self.config.minimum_redundancy, self.config.name))
+                self.config.minimum_redundancy = 0
+        else:
+            _logger.warning('Invalid minimum_redundancy "%s" for server "%s" (fallback to "0")'
+                            % (self.config.minimum_redundancy, self.config.name))
+            self.config.minimum_redundancy = 0
+
+        # Initialise retention policies
+        self._init_retention_policies()
+
+
+    def _init_retention_policies(self):
+
+        # Set retention policy mode
+        if self.config.retention_policy_mode != 'auto':
+            _logger.warning('Unsupported retention_policy_mode "%s" for server "%s" (fallback to "auto")'
+                            % (self.config.retention_policy_mode, self.config.name))
+            self.config.retention_policy_mode = 'auto'
+
+        # If retention_policy is present, enforce them
+        if self.config.retention_policy:
+            # Check wal_retention_policy
+            if self.config.wal_retention_policy != 'main':
+                _logger.warning('Unsupported wal_retention_policy value "%s" for server "%s" (fallback to "main")'
+                                % (self.config.wal_retention_policy, self.config.name))
+                self.config.wal_retention_policy = 'main'
+            # Create retention policy objects
+            try:
+                rp = RetentionPolicyFactory.create(self,
+                    'retention_policy', self.config.retention_policy)
+                # Reassign the configuration value (we keep it in one place)
+                self.config.retention_policy = rp
+                _logger.info('Retention policy for server %s: %s' % (
+                    self.config.name, self.config.retention_policy))
+                try:
+                    rp = RetentionPolicyFactory.create(self,
+                        'wal_retention_policy', self.config.wal_retention_policy)
+                    # Reassign the configuration value (we keep it in one place)
+                    self.wal_retention_policy = rp
+                    _logger.info('WAL retention policy for server %s: %s' % (
+                        self.config.name, self.config.wal_retention_policy))
+                except:
+                    _logger.error('Invalid wal_retention_policy setting "%s" for server "%s" (fallback to "main")' % (
+                        self.config.wal_retention_policy, self.config.name))
+                    self.wal_retention_policy = SimpleWALRetentionPolicy (
+                        self.retention_policy, self)
+
+                self.enforce_retention_policies = True
+
+            except:
+                _logger.error('Invalid retention_policy setting "%s" for server "%s"' % (
+                    self.config.retention_policy, self.config.name))
+
+    def check_retention_policy_settings(self):
+        '''Checks retention policy settings'''
+        if self.config.retention_policy and not self.enforce_retention_policies:
+            yield ("\tretention policy settings: FAILED (see log)", False)
+        else:
+            yield ("\tretention policy settings: OK", True)
 
     def check_ssh(self):
         '''Checks SSH connection'''
@@ -103,6 +198,8 @@ class Server(object):
         status.append(self.check_ssh())
         status.append(self.check_postgres())
         status.append(self.check_directories())
+        # Check retention policies
+        status.append(self.check_retention_policy_settings())
         # Executes the backup manager set of checks
         status.append(self.backup_manager.check())
 
@@ -129,11 +226,21 @@ class Server(object):
         if remote_status['current_xlog']:
             yield "\tcurrent_xlog: %s " % (remote_status['current_xlog'])
 
+    def status_retention_policies(self):
+        '''Status of retention policies enforcement'''
+        if self.enforce_retention_policies:
+            yield "\tRetention policies: enforced (mode: %s, retention: %s, WAL retention: %s)" % (
+                self.config.retention_policy_mode, self.config.retention_policy,
+                self.config.wal_retention_policy)
+        else:
+            yield "\tRetention policies: not enforced"
+
     def status(self):
         '''Implements the 'server status' command.'''
         status = [("Server %s:" % (self.config.name),)]
         if self.config.description: status.append(("\tdescription: %s" % (self.config.description),))
         status.append(self.status_postgres())
+        status.append(self.status_retention_policies())
         # Executes the backup manager status info method
         status.append(self.backup_manager.status())
         return itertools.chain.from_iterable(status)
@@ -165,7 +272,7 @@ class Server(object):
         result['last_shipped_wal'] = None
         if result['data_directory'] and result['archive_command']:
             archive_dir = os.path.join(result['data_directory'], 'pg_xlog', 'archive_status')
-            out = cmd.getoutput(None, 'ls', '-tr', archive_dir)[0]
+            out = str(cmd.getoutput(None, 'ls', '-tr', archive_dir)[0])
             for line in out.splitlines():
                 if line.endswith('.done'):
                     name = line[:-5]
@@ -308,6 +415,7 @@ class Server(object):
     def list_backups(self):
         '''Lists all the available backups for the server'''
         status_filter = BackupInfo.STATUS_NOT_EMPTY
+        retention_status = self.report_backups()
         backups = self.get_available_backups(status_filter)
         for key in sorted(backups.iterkeys(), reverse=True):
             backup = backups[key]
@@ -316,16 +424,19 @@ class Server(object):
                 backup_size = _pretty_size(backup.size or 0 + backup_wal_size)
                 wal_size = _pretty_size(wal_until_next_size)
                 end_time = backup.end_time.ctime()
+                rstatus = ''
+                if self.enforce_retention_policies and retention_status[backup.backup_id] != BackupInfo.VALID:
+                    rstatus = ' - %s' % retention_status[backup.backup_id]
                 if backup.tablespaces:
                     tablespaces = [("%s:%s" % (name, location))for name, _, location in backup.tablespaces]
-                    yield ("%s %s - %s - Size: %s - WAL Size: %s (tablespaces: %s)"
+                    yield ("%s %s - %s - Size: %s - WAL Size: %s (tablespaces: %s)%s"
                            % (self.config.name, backup.backup_id,
                               end_time, backup_size, wal_size,
-                              ', '.join(tablespaces)))
+                              ', '.join(tablespaces), rstatus))
                 else:
-                    yield ("%s %s - %s - Size: %s - WAL Size: %s"
+                    yield ("%s %s - %s - Size: %s - WAL Size: %s%s"
                            % (self.config.name, backup.backup_id,
-                              end_time, backup_size, wal_size))
+                              end_time, backup_size, wal_size, rstatus))
             else:
                 yield "%s %s - %s" % (self.config.name, backup.backup_id, backup.status)
 
@@ -429,9 +540,9 @@ class Server(object):
                 wal_last = name
         return wal_num, wal_size, wal_until_next_num, wal_until_next_size, wal_last
 
-    def recover(self, backup, dest, tablespaces=[], target_tli=None, target_time=None, target_xid=None, exclusive=False, remote_command=None):
+    def recover(self, backup, dest, tablespaces=[], target_tli=None, target_time=None, target_xid=None, target_name=None, exclusive=False, remote_command=None):
         '''Performs a recovery of a backup'''
-        return self.backup_manager.recover(backup, dest, tablespaces, target_tli, target_time, target_xid, exclusive, remote_command)
+        return self.backup_manager.recover(backup, dest, tablespaces, target_tli, target_time, target_xid, target_name, exclusive, remote_command)
 
     def cron(self, verbose=True):
         '''Maintenance operations
@@ -472,3 +583,9 @@ class Server(object):
             except:
                 raise ValueError("cannot parse line: %r" % (line,))
         return name, int(size), float(stamp), compression
+
+    def report_backups(self):
+        if not self.enforce_retention_policies:
+            return dict()
+        else:
+            return self.config.retention_policy.report()
